package Hooke;

import java.util.*;

public class Block extends PhysicsElement  {
   private static int id=0;  // Block identification
   private final float mass;
   private float width;
   private Vector2D pos_t;
   private Vector2D pos_tPlusDelta;
   private Vector2D speed_t;
   private Vector2D speed_tPlusDelta;
   private List<Spring> springs = new ArrayList<Spring>(); //Resortes que tiene el bloque
   
   private Block(){   // nobody can create a block without state
     super(id++);
     mass=0;
   }
   public Block(float mass, float width, Vector2D position, Vector2D speed){
      super(id++);
      this.mass = mass;
      this.width = width;
      this.pos_t = position;
      this.speed_t = speed;
      this.pos_tPlusDelta = new Vector2D(0.0f,0.0f);
      this.speed_tPlusDelta = new Vector2D(0.0f,0.0f);
   }
   public void attachLeftSpring (Spring spring){
       attachSpring(spring);
   }
   public void attachRightSpring (Spring spring){
       attachSpring(spring);
   }
   public void attachSpring (Spring spring) {
     springs.add(spring);
   }
   public Vector2D getPosition() {
      return pos_t;
   }
   public void computeNextState(double delta_t) {
     Vector2D force = new Vector2D(0d,0d);
     //Se suman todas las fuerzas...
     for(int i = 0; i < springs.size();i++){
         force.sumarVector(springs.get(i).getForce(this));
     }
     Vector2D a = new Vector2D(force.getRadio(),force.getAngulo());
     a.multiplicaEscalar(1/mass);
     pos_tPlusDelta.setXY(pos_t.getComponentX() + speed_t.getComponentX()*delta_t + a.getComponentX()*delta_t*delta_t/2,
             pos_t.getComponentY() + speed_t.getComponentY()*delta_t + a.getComponentY()*delta_t*delta_t/2);
     speed_tPlusDelta.setXY(speed_t.getComponentX() + a.getComponentX()*delta_t,
             speed_t.getComponentY() + a.getComponentY()*delta_t);
   }
   public void updateState(){
     pos_t.setXY(pos_tPlusDelta.getComponentX(), pos_tPlusDelta.getComponentY());
     speed_t.setXY(speed_tPlusDelta.getComponentX(), speed_tPlusDelta.getComponentY());
   }
   public String getDescription() {
      return "Block_" + getId()+": x,y";
   }
   public String getState() {
      return getPosition()+"";
   }
}
